#pragma once

#include "TipApplication.h"
#include "UpdaterModel.h"
#include "UpdaterView.h"
#include "TipControl.h"

#include <atlpath.h>
#include "tipupdater_dll.h"

#include "Command/InitCommand.h"
#include "../../../common/ConfigXmlParser.h"

//#include "../../../../lib/NELO/include/NELO.h"
#include "../svn_Info.h"

namespace tipupdater
{
const TCHAR NTALKUPDATER_GUID[] = _T("678F4EA5-8292-4dff-8202-D5F8EEAD66E2");

class TIPUPDATER_DLL_API UpdaterMVCFactory : public IMVCFactory
{
public:
	UpdaterMVCFactory() {}
	virtual ~UpdaterMVCFactory() {}

	//virtual override
	AbstractModel* CreateModel()
	{
		return new UpdaterModel;
	}
	AbstractView* CreateView()
	{
		return new UpdaterView;
	}
	AbstractControl* CreateControl()
	{
		return new TipControl;
	}

	void DestroyModel(AbstractModel* model)
	{
		if (model != NULL)
			delete model;
	}
	void DestroyView(AbstractView* view)
	{
		if (view != NULL)
			delete view;
	}
	void DestroyControl(AbstractControl* control)
	{
		if (control != NULL)
			delete control;
	}
	BOOL GetResourcePath(TCHAR* path, int lenPath)
	{
		TCHAR szApplicationPath[_MAX_PATH] = {0};
		GetAppRootDir(szApplicationPath, _MAX_PATH);
		//ATLPath::RemoveFileSpec(szApplicationPath);
 
		TCHAR szResourcePath[_MAX_PATH] = {0};
		_tcsncpy_s(szResourcePath, szApplicationPath, _MAX_PATH);

		Application* app = Application::get_instance();
		UpdaterModel* model = dynamic_cast<UpdaterModel*>(app->GetModel());
		TipAppDataManager* appData = model->GetAppDataManager();
		CString resourceFileName = appData->GetProperty(tipcommon::Element::ResourceFileName);

 //		TCHAR resourceFileName[tipbase::MAXURL] = {0, };
 //		const TCHAR CONFIG_XML_FILENAME[] = _T("NToolsUpdater.xml");
 //		if (FALSE == tipcommon::ConfigXmlParser::GetURL(CONFIG_XML_FILENAME, tipcommon::Element::ResourceFileName, resourceFileName, tipbase::MAXURL))
 //			return FALSE;

		ATLPath::Append(szResourcePath, resourceFileName);

		_tcsncpy_s(path, lenPath, szResourcePath, _MAX_PATH);
		return TRUE;
	}

	BOOL GetAppRootDir(TCHAR* pDir, int nMax)
	{
		ZeroMemory(pDir, nMax);				
		if(GetCurrentDirectoryW(nMax, pDir))
		{			
			_tcscat(pDir, _T("\\"));
			return TRUE;
		}
		else
		{
			return FALSE;
		}
	}
};

class TIPUPDATER_DLL_API UpdaterApplication : public TipApplication
{
public:
	UpdaterApplication(std::vector<CString>* cmdParameter):m_MVCFactory(new UpdaterMVCFactory()),m_CmdParameter(cmdParameter)
	{
		CString strRevisionNumber;
		strRevisionNumber.Format(_T("%d%d"), 0, CSVNInfo::Get_SVN_Rev());
		OutputDebugString(strRevisionNumber);
		//g_hNELO = NELO_Create(_T("ntalk_upgrader"), strRevisionNumber, _T("Ntalk Upgrader"));
		//NELO_SetReportServer(g_hNELO, _T("ecs.naver.com"), 10007);
		//if(!NELO_StartCrashHandler(g_hNELO))
		//{
		//	int nErrcode = NELO_GetLastError();			
		//}
		//NELO_SetAutoReport(g_hNELO, true);		
	}
	virtual ~UpdaterApplication(void)
	{		
		//NELO_StopCrashHandler(g_hNELO);
		//NELO_Destroy(g_hNELO);		
	}
	
	BOOL IsSilentRun()
	{
		return IsOptionFound( _T("/M") );
	}

	BOOL IsValidRun()
	{
		return TRUE;
	}

	BOOL IsOptionFound( LPCTSTR szOption )
	{
		BOOL bFound = FALSE;
		if (m_CmdParameter->size())
		{
			int num = m_CmdParameter->size();
			CString strParameter;
			for (int i = 0; i < num; i++)
			{
				strParameter = m_CmdParameter->at(i);
				if (0 == strParameter.CompareNoCase(szOption))
					bFound = TRUE;
			}
		}
		return bFound;
	}

	BOOL IsForceRestart()
	{
		return IsOptionFound( _T("/ForceRestart") );
	}
protected:
	IMVCFactory* GetMVCFactory() { return m_MVCFactory.get(); }
	virtual BOOL PreMessageLoop();
	virtual BOOL PostMessageLoop();

private:
	std::auto_ptr<UpdaterMVCFactory>	m_MVCFactory;	
	std::vector<CString>*				m_CmdParameter;
	
	
	
};

};